@@ -2,7 +2,7 @@
 -- Written by Waldemar Celes
 -- TeCGraf/PUC-Rio
 -- Jul 1998
--- $Id: verbatim.lua,v 1.1 2001/10/27 19:35:29 angband Exp $
+-- $Id: verbatim.lua,v 1.2 2003/08/10 11:43:32 rr9 Exp $
 
 -- This code is free software; you can redistribute it and/or modify it.
 -- The software provided hereunder is on an "as is" basis, and
@@ -17,20 +17,21 @@
 --   line = line text
 classVerbatim = {
  line = '',
- _base = classFeature,
+	cond = nil,    -- condition: where to generate the code (s=suport, r=register)
 }
-settag(classVerbatim,tolua_tag)
+classVerbatim.__index = classVerbatim
+setmetatable(classVerbatim,classFeature)
 
 -- preamble verbatim
 function classVerbatim:preamble ()
- if not self.cond then
+ if self.cond == '' then
   write(self.line)
  end
 end
 
 -- support code
 function classVerbatim:supcode ()
- if self.cond then
+ if strfind(self.cond,'s') then
   write(self.line)
   write('\n')
  end
@@ -38,7 +39,7 @@ end
 
 -- register code
 function classVerbatim:register ()
- if self.cond then
+ if strfind(self.cond,'r') then
   write(self.line)
  end
 end
@@ -54,23 +55,21 @@ end
 
 -- Internal constructor
 function _Verbatim (t)
- t._base = classVerbatim
- settag(t,tolua_tag)
+ setmetatable(t,classVerbatim)
  append(t)
  return t
 end
 
 -- Constructor
 -- Expects a string representing the text line
-function Verbatim (l)
- local c
+function Verbatim (l,cond)
  if strsub(l,1,1) == '$' then
-  c = 1
+  cond = 'sr'       -- generates in both suport and register fragments
   l = strsub(l,2)
  end
  return _Verbatim {
   line = l,
-  cond = c
+  cond = cond or '',
  }
 end
 
