@@ -157,6 +157,7 @@ bool make_attack_normal(int m_idx)
 	{
 		bool visible = FALSE;
 		bool obvious = FALSE;
+		bool do_break = FALSE;
 
 		int power = 0;
 		int damage = 0;
@@ -481,6 +482,8 @@ bool make_attack_normal(int m_idx)
 
 				case RBE_UN_POWER:
 				{
+					int drained = 0;
+
 					/* Take damage */
 					take_hit(damage, ddesc);
 
@@ -496,23 +499,31 @@ bool make_attack_normal(int m_idx)
 						/* Skip non-objects */
 						if (!o_ptr->k_idx) continue;
 
-						/* Drain charged wands/staffs */
-						if (((o_ptr->tval == TV_STAFF) ||
-						     (o_ptr->tval == TV_WAND)) &&
-						    (o_ptr->pval > 0))
+						/* Drain charged wands/staves */
+						if ((o_ptr->tval == TV_STAFF) ||
+						    (o_ptr->tval == TV_WAND))
 						{
-							/* Calculate healed hitpoints */
-							int heal = rlev * o_ptr->pval * o_ptr->number;
+							/* Charged? */
+							if (o_ptr->pval)
+							{
+								drained = o_ptr->pval;
+
+								/* Uncharge */
+								o_ptr->pval = 0;
+							}
+						}
+
+						if (drained)
+						{
+							int heal = rlev * drained;
+
+							msg_print("Energy drains from your pack!");
+
+							obvious = TRUE;
 
 							/* Don't heal more than max hp */
 							heal = MIN(heal, m_ptr->maxhp - m_ptr->hp);
 
-							/* Message */
-							msg_print("Energy drains from your pack!");
-
-							/* Obvious */
-							obvious = TRUE;
-
 							/* Heal */
 							m_ptr->hp += heal;
 
@@ -520,16 +531,13 @@ bool make_attack_normal(int m_idx)
 							if (p_ptr->health_who == m_idx)
 								p_ptr->redraw |= (PR_HEALTH);
 
-							/* Uncharge */
-							o_ptr->pval = 0;
-
 							/* Combine / Reorder the pack */
 							p_ptr->notice |= (PN_COMBINE | PN_REORDER);
 
 							/* Window stuff */
 							p_ptr->window |= (PW_INVEN);
 
-							/* Done */
+							/* Affect only a single inventory slot */
 							break;
 						}
 					}
@@ -651,6 +659,12 @@ bool make_attack_normal(int m_idx)
 						/* Modify number */
 						i_ptr->number = 1;
 
+						/* Hack -- If a rod, staff, or wand, allocate total
+						 * maximum timeouts or charges between those
+						 * stolen and those missed. -LM-
+						 */
+						distribute_charges(o_ptr, i_ptr, 1);
+
 						/* Carry the object */
 						(void)monster_carry(m_idx, i_ptr);
 
@@ -1007,7 +1021,18 @@ bool make_attack_normal(int m_idx)
 					take_hit(damage, ddesc);
 
 					/* Radius 8 earthquake centered at the monster */
-					if (damage > 23) earthquake(m_ptr->fy, m_ptr->fx, 8);
+					if (damage > 23)
+					{
+						int px_old = p_ptr->px;
+						int py_old = p_ptr->py;
+						
+						earthquake(m_ptr->fy, m_ptr->fx, 8);
+
+						/* Stop the blows if the player is pushed away */
+						if ((px_old != p_ptr->px) ||
+						    (py_old != p_ptr->py))
+						    do_break = TRUE;
+					}
 
 					break;
 				}
@@ -1267,6 +1292,9 @@ bool make_attack_normal(int m_idx)
 				}
 			}
 		}
+
+		/* Skip the other blows if necessary */
+		if (do_break) break;
 	}
 
 
