@@ -747,12 +747,12 @@ s16b get_obj_num(int level)
  *
  * Note that "knowing" an object gives you everything that an "awareness"
  * gives you, and much more.  In fact, the player is always "aware" of any
- * item of which he has full "knowledge".
+ * item which he "knows", except items in stores.
  *
- * But having full knowledge of, say, one "wand of wonder", does not, by
- * itself, give you knowledge, or even awareness, of other "wands of wonder".
+ * But having knowledge of, say, one "wand of wonder", does not, by itself,
+ * give you knowledge, or even awareness, of other "wands of wonder".
  * It happens that most "identify" routines (including "buying from a shop")
- * will make the player "aware" of the object as well as fully "know" it.
+ * will make the player "aware" of the object as well as "know" it.
  *
  * This routine also removes any inscriptions generated by "feelings".
  */
@@ -985,8 +985,8 @@ static s32b object_value_real(const object_type *o_ptr)
 		case TV_WAND:
 		case TV_STAFF:
 		{
-			/* Pay extra for charges */
-			value += ((value / 20) * o_ptr->pval);
+			/* Pay extra for charges, depending on standard number of charges */
+			value += ((value / 20) * (o_ptr->pval / o_ptr->number));
 
 			/* Done */
 			break;
@@ -1147,18 +1147,15 @@ s32b object_value(const object_type *o_ptr)
  *
  * See "object_absorb()" for the actual "absorption" code.
  *
- * If permitted, we allow wands/staffs (if they are known to have equal
- * charges) and rods (if fully charged) to combine.  They will unstack
- * (if necessary) when they are used.
- *
- * If permitted, we allow weapons/armor to stack, if fully "known".
+ * If permitted, we allow weapons/armor to stack, if "known".
  *
  * Missiles will combine if both stacks have the same "known" status.
  * This is done to make unidentified stacks of missiles useful.
  *
  * Food, potions, scrolls, and "easy know" items always stack.
  *
- * Chests, and activatable items, never stack (for various reasons).
+ * Chests, and activatable items, except rods, never stack (for various
+ * reasons).
  */
 bool object_similar(const object_type *o_ptr, const object_type *j_ptr)
 {
@@ -1188,23 +1185,24 @@ bool object_similar(const object_type *o_ptr, const object_type *j_ptr)
 			break;
 		}
 
-		/* Staffs and Wands */
+		/* Staves and Wands */
 		case TV_STAFF:
 		case TV_WAND:
 		{
-			/* Require knowledge */
-			if (!object_known_p(o_ptr) || !object_known_p(j_ptr)) return (0);
+			/* Require either knowledge or known empty for both wands/staves */
+			if ((!(o_ptr->ident & (IDENT_EMPTY)) &&
+				!object_known_p(o_ptr)) ||
+				(!(j_ptr->ident & (IDENT_EMPTY)) &&
+				!object_known_p(j_ptr))) return(0);
 
-			/* Fall through */
+			/* Assume okay */
+			break;
 		}
 
-		/* Staffs and Wands and Rods */
+		/* Rods */
 		case TV_ROD:
 		{
-			/* Require identical charges */
-			if (o_ptr->pval != j_ptr->pval) return (0);
-
-			/* Probably okay */
+			/* Assume okay */
 			break;
 		}
 
@@ -1232,7 +1230,7 @@ bool object_similar(const object_type *o_ptr, const object_type *j_ptr)
 		case TV_AMULET:
 		case TV_LITE:
 		{
-			/* Require full knowledge of both items */
+			/* Require both items to be known */
 			if (!object_known_p(o_ptr) || !object_known_p(j_ptr)) return (0);
 
 			/* Fall through */
@@ -1364,6 +1362,8 @@ bool object_similar(const object_type *o_ptr, const object_type *j_ptr)
  */
 void object_absorb(object_type *o_ptr, const object_type *j_ptr)
 {
+	object_kind *k_ptr = &k_info[o_ptr->k_idx];
+
 	int total = o_ptr->number + j_ptr->number;
 
 	/* Add together the item counts */
@@ -1383,6 +1383,22 @@ void object_absorb(object_type *o_ptr, const object_type *j_ptr)
 
 	/* Mega-Hack -- Blend "discounts" */
 	if (o_ptr->discount < j_ptr->discount) o_ptr->discount = j_ptr->discount;
+
+	/*
+	 * Hack -- if rods are stacking, re-calculate the
+	 * pvals (maximum timeouts) and current timeouts together
+	 */
+	if (o_ptr->tval == TV_ROD)
+	{
+		o_ptr->pval = total * k_ptr->pval;
+		o_ptr->timeout += j_ptr->timeout;
+	}
+
+	/* Hack -- if wands or staves are stacking, combine the charges */
+	if ((o_ptr->tval == TV_WAND) || (o_ptr->tval == TV_STAFF))
+	{
+		o_ptr->pval += j_ptr->pval;
+	}
 }
 
 
@@ -2526,6 +2542,15 @@ static void a_m_aux_4(object_type *o_ptr, int level, int power)
 			break;
 		}
 
+		case TV_ROD:
+		{
+			object_kind *k_ptr = &k_info[o_ptr->k_idx];
+
+			/* Transfer the pval. */
+			o_ptr->pval = k_ptr->pval;
+			break;
+		}
+
 		case TV_CHEST:
 		{
 			/* Hack -- skip ruined chests */
@@ -3959,20 +3984,6 @@ s16b inven_carry(object_type *o_ptr)
 			if (!object_known_p(o_ptr)) continue;
 			if (!object_known_p(j_ptr)) break;
 
-			/* Rods sort by increasing recharge time */
-			if (o_ptr->tval == TV_ROD)
-			{
-				if (o_ptr->pval < j_ptr->pval) break;
-				if (o_ptr->pval > j_ptr->pval) continue;
-			}
-
-			/* Wands/Staffs sort by decreasing charges */
-			if ((o_ptr->tval == TV_WAND) || (o_ptr->tval == TV_STAFF))
-			{
-				if (o_ptr->pval > j_ptr->pval) break;
-				if (o_ptr->pval < j_ptr->pval) continue;
-			}
-
 			/* Lites sort by decreasing fuel */
 			if (o_ptr->tval == TV_LITE)
 			{
@@ -4170,6 +4181,9 @@ void inven_drop(int item, int amt)
 	/* Obtain local object */
 	object_copy(i_ptr, o_ptr);
 
+	/* Distribute charges of wands, staves, or rods */
+	distribute_charges(o_ptr, i_ptr, amt);
+
 	/* Modify quantity */
 	i_ptr->number = amt;
 
@@ -4326,20 +4340,6 @@ void reorder_pack(void)
 			if (!object_known_p(o_ptr)) continue;
 			if (!object_known_p(j_ptr)) break;
 
-			/* Rods sort by increasing recharge time */
-			if (o_ptr->tval == TV_ROD)
-			{
-				if (o_ptr->pval < j_ptr->pval) break;
-				if (o_ptr->pval > j_ptr->pval) continue;
-			}
-
-			/* Wands/Staffs sort by decreasing charges */
-			if ((o_ptr->tval == TV_WAND) || (o_ptr->tval == TV_STAFF))
-			{
-				if (o_ptr->pval > j_ptr->pval) break;
-				if (o_ptr->pval < j_ptr->pval) continue;
-			}
-
 			/* Lites sort by decreasing fuel */
 			if (o_ptr->tval == TV_LITE)
 			{
@@ -4384,3 +4384,63 @@ void reorder_pack(void)
 	/* Message */
 	if (flag) msg_print("You reorder some items in your pack.");
 }
+
+
+/*
+ * Distribute charges of rods, staves, or wands.
+ *
+ * o_ptr = source item
+ * q_ptr = target item, must be of the same type as o_ptr
+ * amt   = number of items that are transfered
+ */
+void distribute_charges(object_type *o_ptr, object_type *q_ptr, int amt)
+{
+	/*
+	 * Hack -- If rods, staves, or wands are dropped, the total maximum
+	 * timeout or charges need to be allocated between the two stacks.
+	 * If all the items are being dropped, it makes for a neater message
+	 * to leave the original stack's pval alone. -LM-
+	 */
+	if ((o_ptr->tval == TV_WAND) ||
+	    (o_ptr->tval == TV_STAFF) ||
+	    (o_ptr->tval == TV_ROD))
+	{
+		q_ptr->pval = o_ptr->pval * amt / o_ptr->number;
+
+		if (amt < o_ptr->number) o_ptr->pval -= q_ptr->pval;
+
+		/*
+		 * Hack -- Rods also need to have their timeouts distributed.
+		 *
+		 * The dropped stack will accept all time remaining to charge up to
+		 * its maximum.
+		 */
+		if ((o_ptr->tval == TV_ROD) && (o_ptr->timeout))
+		{
+			if (q_ptr->pval > o_ptr->timeout)
+				q_ptr->timeout = o_ptr->timeout;
+			else
+				q_ptr->timeout = q_ptr->pval;
+
+			if (amt < o_ptr->number)
+				o_ptr->timeout -= q_ptr->timeout;
+		}
+	}
+}
+
+
+void reduce_charges(object_type *o_ptr, int amt)
+{
+	/*
+	 * Hack -- If rods or wand are destroyed, the total maximum timeout or
+	 * charges of the stack needs to be reduced, unless all the items are
+	 * being destroyed. -LM-
+	 */
+	if (((o_ptr->tval == TV_WAND) ||
+	     (o_ptr->tval == TV_STAFF) ||
+	     (o_ptr->tval == TV_ROD)) &&
+	    (amt < o_ptr->number))
+	{
+		o_ptr->pval -= o_ptr->pval * amt / o_ptr->number;
+	}
+}
