@@ -50,94 +50,94 @@ const char hexsym[16] =
 
 
 /*
- * Stat Table (INT/WIS) -- Number of half-spells per level
+ * Stat Table (INT/WIS) -- Number of 1/100 spells per level
  */
-const byte adj_mag_study[] =
+const int adj_mag_study[] =
 {
-	0	/* 3 */,
-	0	/* 4 */,
-	0	/* 5 */,
-	0	/* 6 */,
-	0	/* 7 */,
-	1	/* 8 */,
-	1	/* 9 */,
-	1	/* 10 */,
-	1	/* 11 */,
-	2	/* 12 */,
-	2	/* 13 */,
-	2	/* 14 */,
-	2	/* 15 */,
-	2	/* 16 */,
-	2	/* 17 */,
-	2	/* 18/00-18/09 */,
-	2	/* 18/10-18/19 */,
-	2	/* 18/20-18/29 */,
-	2	/* 18/30-18/39 */,
-	2	/* 18/40-18/49 */,
-	3	/* 18/50-18/59 */,
-	3	/* 18/60-18/69 */,
-	3	/* 18/70-18/79 */,
-	3	/* 18/80-18/89 */,
-	4	/* 18/90-18/99 */,
-	4	/* 18/100-18/109 */,
-	4	/* 18/110-18/119 */,
-	5	/* 18/120-18/129 */,
-	5	/* 18/130-18/139 */,
-	5	/* 18/140-18/149 */,
-	5	/* 18/150-18/159 */,
-	5	/* 18/160-18/169 */,
-	5	/* 18/170-18/179 */,
-	5	/* 18/180-18/189 */,
-	5	/* 18/190-18/199 */,
-	5	/* 18/200-18/209 */,
-	5	/* 18/210-18/219 */,
-	5	/* 18/220+ */
+	  0	/* 3 */,
+	  0	/* 4 */,
+	 10	/* 5 */,
+	 20	/* 6 */,
+	 30	/* 7 */,
+	 40	/* 8 */,
+	 50	/* 9 */,
+	 60	/* 10 */,
+	 70	/* 11 */,
+	 80	/* 12 */,
+	 85	/* 13 */,
+	 90	/* 14 */,
+	 95	/* 15 */,
+	100	/* 16 */,
+	105	/* 17 */,
+	110	/* 18/00-18/09 */,
+	115	/* 18/10-18/19 */,
+	120	/* 18/20-18/29 */,
+	130	/* 18/30-18/39 */,
+	140	/* 18/40-18/49 */,
+	150	/* 18/50-18/59 */,
+	160	/* 18/60-18/69 */,
+	170	/* 18/70-18/79 */,
+	180	/* 18/80-18/89 */,
+	190	/* 18/90-18/99 */,
+	200	/* 18/100-18/109 */,
+	210	/* 18/110-18/119 */,
+	220	/* 18/120-18/129 */,
+	230	/* 18/130-18/139 */,
+	240	/* 18/140-18/149 */,
+	250	/* 18/150-18/159 */,
+	250	/* 18/160-18/169 */,
+	250	/* 18/170-18/179 */,
+	250	/* 18/180-18/189 */,
+	250	/* 18/190-18/199 */,
+	250	/* 18/200-18/209 */,
+	250	/* 18/210-18/219 */,
+	250	/* 18/220+ */
 };
 
 
 /*
- * Stat Table (INT/WIS) -- extra half-mana-points per level
+ * Stat Table (INT/WIS) -- extra 1/100 mana-points per level
  */
-const byte adj_mag_mana[] =
+const int adj_mag_mana[] =
 {
-	0	/* 3 */,
-	0	/* 4 */,
-	0	/* 5 */,
-	0	/* 6 */,
-	0	/* 7 */,
-	1	/* 8 */,
-	2	/* 9 */,
-	2	/* 10 */,
-	2	/* 11 */,
-	2	/* 12 */,
-	2	/* 13 */,
-	2	/* 14 */,
-	2	/* 15 */,
-	2	/* 16 */,
-	2	/* 17 */,
-	3	/* 18/00-18/09 */,
-	3	/* 18/10-18/19 */,
-	3	/* 18/20-18/29 */,
-	3	/* 18/30-18/39 */,
-	3	/* 18/40-18/49 */,
-	4	/* 18/50-18/59 */,
-	4	/* 18/60-18/69 */,
-	5	/* 18/70-18/79 */,
-	6	/* 18/80-18/89 */,
-	7	/* 18/90-18/99 */,
-	8	/* 18/100-18/109 */,
-	9	/* 18/110-18/119 */,
-	10	/* 18/120-18/129 */,
-	11	/* 18/130-18/139 */,
-	12	/* 18/140-18/149 */,
-	13	/* 18/150-18/159 */,
-	14	/* 18/160-18/169 */,
-	15	/* 18/170-18/179 */,
-	16	/* 18/180-18/189 */,
-	16	/* 18/190-18/199 */,
-	16	/* 18/200-18/209 */,
-	16	/* 18/210-18/219 */,
-	16	/* 18/220+ */
+	  0	/* 3 */,
+	 10	/* 4 */,
+	 20	/* 5 */,
+	 30	/* 6 */,
+	 40	/* 7 */,
+	 50	/* 8 */,
+	 60	/* 9 */,
+	 70	/* 10 */,
+	 80	/* 11 */,
+	 90	/* 12 */,
+	100	/* 13 */,
+	110	/* 14 */,
+	120	/* 15 */,
+	130	/* 16 */,
+	140	/* 17 */,
+	150	/* 18/00-18/09 */,
+	160	/* 18/10-18/19 */,
+	170	/* 18/20-18/29 */,
+	180	/* 18/30-18/39 */,
+	190	/* 18/40-18/49 */,
+	200	/* 18/50-18/59 */,
+	225	/* 18/60-18/69 */,
+	250	/* 18/70-18/79 */,
+	300	/* 18/80-18/89 */,
+	350	/* 18/90-18/99 */,
+	400	/* 18/100-18/109 */,
+	450	/* 18/110-18/119 */,
+	500	/* 18/120-18/129 */,
+	550	/* 18/130-18/139 */,
+	600	/* 18/140-18/149 */,
+	650	/* 18/150-18/159 */,
+	700	/* 18/160-18/169 */,
+	750	/* 18/170-18/179 */,
+	800	/* 18/180-18/189 */,
+	800	/* 18/190-18/199 */,
+	800	/* 18/200-18/209 */,
+	800	/* 18/210-18/219 */,
+	800	/* 18/220+ */
 };
 
 
@@ -188,48 +188,48 @@ const byte adj_mag_fail[] =
 
 
 /*
- * Stat Table (INT/WIS) -- Various things
+ * Stat Table (INT/WIS) -- failure rate adjustment
  */
-const byte adj_mag_stat[] =
+const int adj_mag_stat[] =
 {
-	0	/* 3 */,
-	0	/* 4 */,
-	0	/* 5 */,
-	0	/* 6 */,
-	0	/* 7 */,
-	1	/* 8 */,
-	1	/* 9 */,
-	1	/* 10 */,
-	1	/* 11 */,
-	1	/* 12 */,
-	1	/* 13 */,
-	1	/* 14 */,
-	2	/* 15 */,
-	2	/* 16 */,
-	2	/* 17 */,
-	3	/* 18/00-18/09 */,
-	3	/* 18/10-18/19 */,
-	3	/* 18/20-18/29 */,
-	3	/* 18/30-18/39 */,
-	3	/* 18/40-18/49 */,
-	4	/* 18/50-18/59 */,
-	4	/* 18/60-18/69 */,
-	5	/* 18/70-18/79 */,
-	6	/* 18/80-18/89 */,
-	7	/* 18/90-18/99 */,
-	8	/* 18/100-18/109 */,
-	9	/* 18/110-18/119 */,
-	10	/* 18/120-18/129 */,
-	11	/* 18/130-18/139 */,
-	12	/* 18/140-18/149 */,
-	13	/* 18/150-18/159 */,
-	14	/* 18/160-18/169 */,
-	15	/* 18/170-18/179 */,
-	16	/* 18/180-18/189 */,
-	17	/* 18/190-18/199 */,
-	18	/* 18/200-18/209 */,
-	19	/* 18/210-18/219 */,
-	20	/* 18/220+ */
+	-5	/* 3 */,
+	-4	/* 4 */,
+	-3	/* 5 */,
+	-3	/* 6 */,
+	-2	/* 7 */,
+	-1	/* 8 */,
+	 0	/* 9 */,
+	 0	/* 10 */,
+	 0	/* 11 */,
+	 0	/* 12 */,
+	 0	/* 13 */,
+	 1	/* 14 */,
+	 2	/* 15 */,
+	 3	/* 16 */,
+	 4	/* 17 */,
+	 5	/* 18/00-18/09 */,
+	 6	/* 18/10-18/19 */,
+	 7	/* 18/20-18/29 */,
+	 8	/* 18/30-18/39 */,
+	 9	/* 18/40-18/49 */,
+	10	/* 18/50-18/59 */,
+	11	/* 18/60-18/69 */,
+	12	/* 18/70-18/79 */,
+	15	/* 18/80-18/89 */,
+	18	/* 18/90-18/99 */,
+	21	/* 18/100-18/109 */,
+	24	/* 18/110-18/119 */,
+	27	/* 18/120-18/129 */,
+	30	/* 18/130-18/139 */,
+	33	/* 18/140-18/149 */,
+	36	/* 18/150-18/159 */,
+	39	/* 18/160-18/169 */,
+	42	/* 18/170-18/179 */,
+	45	/* 18/180-18/189 */,
+	48	/* 18/190-18/199 */,
+	51	/* 18/200-18/209 */,
+	54	/* 18/210-18/219 */,
+	57	/* 18/220+ */
 };
 
 
@@ -970,48 +970,48 @@ const byte adj_con_fix[] =
 
 
 /*
- * Stat Table (CON) -- extra half-hitpoints per level (plus 128)
+ * Stat Table (CON) -- extra 1/100th hitpoints per level
  */
-const byte adj_con_mhp[] =
+const int adj_con_mhp[] =
 {
-	128 + -5	/* 3 */,
-	128 + -3	/* 4 */,
-	128 + -2	/* 5 */,
-	128 + -1	/* 6 */,
-	128 + 0	/* 7 */,
-	128 + 0	/* 8 */,
-	128 + 0	/* 9 */,
-	128 + 0	/* 10 */,
-	128 + 0	/* 11 */,
-	128 + 0	/* 12 */,
-	128 + 0	/* 13 */,
-	128 + 0	/* 14 */,
-	128 + 1	/* 15 */,
-	128 + 1	/* 16 */,
-	128 + 2	/* 17 */,
-	128 + 3	/* 18/00-18/09 */,
-	128 + 4	/* 18/10-18/19 */,
-	128 + 4	/* 18/20-18/29 */,
-	128 + 4	/* 18/30-18/39 */,
-	128 + 4	/* 18/40-18/49 */,
-	128 + 5	/* 18/50-18/59 */,
-	128 + 6	/* 18/60-18/69 */,
-	128 + 7	/* 18/70-18/79 */,
-	128 + 8	/* 18/80-18/89 */,
-	128 + 9	/* 18/90-18/99 */,
-	128 + 10	/* 18/100-18/109 */,
-	128 + 11	/* 18/110-18/119 */,
-	128 + 12	/* 18/120-18/129 */,
-	128 + 13	/* 18/130-18/139 */,
-	128 + 14	/* 18/140-18/149 */,
-	128 + 15	/* 18/150-18/159 */,
-	128 + 16	/* 18/160-18/169 */,
-	128 + 18	/* 18/170-18/179 */,
-	128 + 20	/* 18/180-18/189 */,
-	128 + 22	/* 18/190-18/199 */,
-	128 + 25	/* 18/200-18/209 */,
-	128 + 25	/* 18/210-18/219 */,
-	128 + 25	/* 18/220+ */
+	-250	/* 3 */,
+	-150	/* 4 */,
+	-100	/* 5 */,
+	 -75	/* 6 */,
+	 -50	/* 7 */,
+	 -25	/* 8 */,
+	 -10	/* 9 */,
+	  -5	/* 10 */,
+	   0	/* 11 */,
+	   5	/* 12 */,
+	  10	/* 13 */,
+	  25	/* 14 */,
+	  50	/* 15 */,
+	  75	/* 16 */,
+	 100	/* 17 */,
+	 150	/* 18/00-18/09 */,
+	 175	/* 18/10-18/19 */,
+	 200	/* 18/20-18/29 */,
+	 225	/* 18/30-18/39 */,
+	 250	/* 18/40-18/49 */,
+	 275	/* 18/50-18/59 */,
+	 300	/* 18/60-18/69 */,
+	 350	/* 18/70-18/79 */,
+	 400	/* 18/80-18/89 */,
+	 450	/* 18/90-18/99 */,
+	 500	/* 18/100-18/109 */,
+	 550	/* 18/110-18/119 */,
+	 600	/* 18/120-18/129 */,
+	 650	/* 18/130-18/139 */,
+	 700	/* 18/140-18/149 */,
+	 750	/* 18/150-18/159 */,
+	 800	/* 18/160-18/169 */,
+	 900	/* 18/170-18/179 */,
+	1000	/* 18/180-18/189 */,
+	1100	/* 18/190-18/199 */,
+	1250	/* 18/200-18/209 */,
+	1250	/* 18/210-18/219 */,
+	1250	/* 18/220+ */
 };
 
 
@@ -1366,7 +1366,7 @@ cptr window_flag_desc[32] =
 	"Display overhead view",
 	"Display monster recall",
 	"Display object recall",
-	NULL,
+	"Display monster list",
 	"Display snap-shot",
 	"Display script variables",
 	"Display script source",
@@ -1428,7 +1428,7 @@ cptr option_text[OPT_MAX] =
 	"verify_special",			/* OPT_verify_special */
 	"allow_quantity",			/* OPT_allow_quantity */
 	NULL,						/* xxx */
-	"auto_haggle",				/* OPT_auto_haggle */
+	NULL,						/* xxx auto_haggle */
 	"auto_scum",				/* OPT_auto_scum */
 	NULL,						/* xxx testing_stack */
 	NULL,						/* xxx testing_carry */
@@ -1692,7 +1692,7 @@ cptr option_desc[OPT_MAX] =
 	"Verify use of special commands",			/* OPT_verify_special */
 	"Allow quantity specification",				/* OPT_allow_quantity */
 	NULL,										/* xxx */
-	"Auto-haggle in stores",					/* OPT_auto_haggle */
+	NULL,										/* xxx auto_haggle */
 	"Auto-scum for good levels",				/* OPT_auto_scum */
 	NULL,										/* xxx testing_stack */
 	NULL,										/* xxx testing_carry */
@@ -1956,7 +1956,7 @@ const bool option_norm[OPT_MAX] =
 	TRUE,		/* OPT_verify_special */
 	TRUE,		/* OPT_allow_quantity */
 	FALSE,		/* xxx */
-	TRUE,		/* OPT_auto_haggle */
+	FALSE,		/* xxx auto_haggle */
 	FALSE,		/* OPT_auto_scum */
 	FALSE,		/* xxx */
 	FALSE,		/* xxx */
@@ -2241,7 +2241,6 @@ const byte option_page[OPT_PAGE_MAX][OPT_PAGE_PER] =
 	/*** Game-Play ***/
 
 	{
-		OPT_auto_haggle,
 		OPT_auto_scum,
 		OPT_expand_look,
 		OPT_expand_list,
@@ -2260,6 +2259,7 @@ const byte option_page[OPT_PAGE_MAX][OPT_PAGE_PER] =
 		OPT_NONE,
 		OPT_NONE,
 		OPT_NONE,
+		OPT_NONE,
 		OPT_NONE
 	},
 
