@@ -991,7 +991,7 @@ static bool read_scroll(object_type *o_ptr, bool *ident)
 
 		case SV_SCROLL_BANISHMENT:
 		{
-			(void)banishment();
+			if (!banishment()) used_up = FALSE;
 			*ident = TRUE;
 			break;
 		}
@@ -1255,7 +1255,7 @@ static bool use_staff(object_type *o_ptr, bool *ident)
 
 		case SV_STAFF_BANISHMENT:
 		{
-			(void)banishment();
+			if (!banishment()) use_charge = FALSE;
 			*ident = TRUE;
 			break;
 		}
@@ -1573,6 +1573,9 @@ static bool aim_wand(object_type *o_ptr, bool *ident)
 static bool zap_rod(object_type *o_ptr, bool *ident)
 {
 	int chance, dir, lev;
+	bool used_charge = TRUE;
+	object_kind *k_ptr = &k_info[o_ptr->k_idx];
+
 
 	/* Get a direction (unless KNOWN not to need it) */
 	if ((o_ptr->sval >= SV_ROD_MIN_DIRECTION) || !object_aware_p(o_ptr))
@@ -1614,26 +1617,28 @@ static bool zap_rod(object_type *o_ptr, bool *ident)
 		return FALSE;
 	}
 
-	/* Still charging */
-	if (o_ptr->pval)
+	/* Still charging? */
+	if (o_ptr->timeout > (o_ptr->pval - k_ptr->pval))
 	{
 		if (flush_failure) flush();
-		msg_print("The rod is still charging.");
+
+		if (o_ptr->number == 1)
+			msg_print("The rod is still charging");
+		else
+			msg_print("The rods are all still charging");
+
 		return FALSE;
 	}
 
-
 	/* Sound */
 	sound(MSG_ZAP);
 
-
 	/* Analyze the rod */
 	switch (o_ptr->sval)
 	{
 		case SV_ROD_DETECT_TRAP:
 		{
 			if (detect_traps()) *ident = TRUE;
-			o_ptr->pval = 50;
 			break;
 		}
 
@@ -1641,14 +1646,13 @@ static bool zap_rod(object_type *o_ptr, bool *ident)
 		{
 			if (detect_doors()) *ident = TRUE;
 			if (detect_stairs()) *ident = TRUE;
-			o_ptr->pval = 70;
 			break;
 		}
 
 		case SV_ROD_IDENTIFY:
 		{
 			*ident = TRUE;
-			if (ident_spell()) o_ptr->pval = 10;
+			if (ident_spell()) used_charge = FALSE;
 			break;
 		}
 
@@ -1656,14 +1660,12 @@ static bool zap_rod(object_type *o_ptr, bool *ident)
 		{
 			set_recall();
 			*ident = TRUE;
-			o_ptr->pval = 60;
 			break;
 		}
 
 		case SV_ROD_ILLUMINATION:
 		{
 			if (lite_area(damroll(2, 8), 2)) *ident = TRUE;
-			o_ptr->pval = 30;
 			break;
 		}
 
@@ -1671,7 +1673,6 @@ static bool zap_rod(object_type *o_ptr, bool *ident)
 		{
 			map_area();
 			*ident = TRUE;
-			o_ptr->pval = 99;
 			break;
 		}
 
@@ -1679,7 +1680,6 @@ static bool zap_rod(object_type *o_ptr, bool *ident)
 		{
 			detect_all();
 			*ident = TRUE;
-			o_ptr->pval = 99;
 			break;
 		}
 
@@ -1687,7 +1687,6 @@ static bool zap_rod(object_type *o_ptr, bool *ident)
 		{
 			probing();
 			*ident = TRUE;
-			o_ptr->pval = 50;
 			break;
 		}
 
@@ -1698,7 +1697,6 @@ static bool zap_rod(object_type *o_ptr, bool *ident)
 			if (set_confused(0)) *ident = TRUE;
 			if (set_stun(0)) *ident = TRUE;
 			if (set_cut(0)) *ident = TRUE;
-			o_ptr->pval = 999;
 			break;
 		}
 
@@ -1707,7 +1705,6 @@ static bool zap_rod(object_type *o_ptr, bool *ident)
 			if (hp_player(500)) *ident = TRUE;
 			if (set_stun(0)) *ident = TRUE;
 			if (set_cut(0)) *ident = TRUE;
-			o_ptr->pval = 999;
 			break;
 		}
 
@@ -1720,7 +1717,6 @@ static bool zap_rod(object_type *o_ptr, bool *ident)
 			if (do_res_stat(A_DEX)) *ident = TRUE;
 			if (do_res_stat(A_CON)) *ident = TRUE;
 			if (do_res_stat(A_CHR)) *ident = TRUE;
-			o_ptr->pval = 999;
 			break;
 		}
 
@@ -1734,21 +1730,18 @@ static bool zap_rod(object_type *o_ptr, bool *ident)
 			{
 				(void)set_fast(p_ptr->fast + 5);
 			}
-			o_ptr->pval = 99;
 			break;
 		}
 
 		case SV_ROD_TELEPORT_AWAY:
 		{
 			if (teleport_monster(dir)) *ident = TRUE;
-			o_ptr->pval = 25;
 			break;
 		}
 
 		case SV_ROD_DISARMING:
 		{
 			if (disarm_trap(dir)) *ident = TRUE;
-			o_ptr->pval = 30;
 			break;
 		}
 
@@ -1757,35 +1750,30 @@ static bool zap_rod(object_type *o_ptr, bool *ident)
 			msg_print("A line of blue shimmering light appears.");
 			lite_line(dir);
 			*ident = TRUE;
-			o_ptr->pval = 9;
 			break;
 		}
 
 		case SV_ROD_SLEEP_MONSTER:
 		{
 			if (sleep_monster(dir)) *ident = TRUE;
-			o_ptr->pval = 18;
 			break;
 		}
 
 		case SV_ROD_SLOW_MONSTER:
 		{
 			if (slow_monster(dir)) *ident = TRUE;
-			o_ptr->pval = 20;
 			break;
 		}
 
 		case SV_ROD_DRAIN_LIFE:
 		{
 			if (drain_life(dir, 150)) *ident = TRUE;
-			o_ptr->pval = 23;
 			break;
 		}
 
 		case SV_ROD_POLYMORPH:
 		{
 			if (poly_monster(dir)) *ident = TRUE;
-			o_ptr->pval = 25;
 			break;
 		}
 
@@ -1793,7 +1781,6 @@ static bool zap_rod(object_type *o_ptr, bool *ident)
 		{
 			fire_bolt_or_beam(10, GF_ACID, dir, damroll(12, 8));
 			*ident = TRUE;
-			o_ptr->pval = 12;
 			break;
 		}
 
@@ -1801,7 +1788,6 @@ static bool zap_rod(object_type *o_ptr, bool *ident)
 		{
 			fire_bolt_or_beam(10, GF_ELEC, dir, damroll(6, 6));
 			*ident = TRUE;
-			o_ptr->pval = 11;
 			break;
 		}
 
@@ -1809,7 +1795,6 @@ static bool zap_rod(object_type *o_ptr, bool *ident)
 		{
 			fire_bolt_or_beam(10, GF_FIRE, dir, damroll(16, 8));
 			*ident = TRUE;
-			o_ptr->pval = 15;
 			break;
 		}
 
@@ -1817,7 +1802,6 @@ static bool zap_rod(object_type *o_ptr, bool *ident)
 		{
 			fire_bolt_or_beam(10, GF_COLD, dir, damroll(10, 8));
 			*ident = TRUE;
-			o_ptr->pval = 13;
 			break;
 		}
 
@@ -1825,7 +1809,6 @@ static bool zap_rod(object_type *o_ptr, bool *ident)
 		{
 			fire_ball(GF_ACID, dir, 120, 2);
 			*ident = TRUE;
-			o_ptr->pval = 27;
 			break;
 		}
 
@@ -1833,7 +1816,6 @@ static bool zap_rod(object_type *o_ptr, bool *ident)
 		{
 			fire_ball(GF_ELEC, dir, 64, 2);
 			*ident = TRUE;
-			o_ptr->pval = 23;
 			break;
 		}
 
@@ -1841,7 +1823,6 @@ static bool zap_rod(object_type *o_ptr, bool *ident)
 		{
 			fire_ball(GF_FIRE, dir, 144, 2);
 			*ident = TRUE;
-			o_ptr->pval = 30;
 			break;
 		}
 
@@ -1849,11 +1830,13 @@ static bool zap_rod(object_type *o_ptr, bool *ident)
 		{
 			fire_ball(GF_COLD, dir, 96, 2);
 			*ident = TRUE;
-			o_ptr->pval = 25;
 			break;
 		}
 	}
 
+	/* Drain the charge */
+	if (used_charge) o_ptr->timeout += k_ptr->pval;
+
 	return TRUE;
 }
 
@@ -2021,7 +2004,7 @@ static bool activate_object(object_type *o_ptr, bool *ident)
 			case ACT_BANISHMENT:
 			{
 				msg_format("Your %s glows deep blue...", o_name);
-				(void)banishment();
+				if (!banishment()) return FALSE;
 				break;
 			}
 
@@ -2665,6 +2648,38 @@ void describe_item_activation(const object_type *o_ptr)
 		return;
 	}
 
+	/* Ring activations */
+	if (o_ptr->tval == TV_RING)
+	{
+		/* Branch on the sub-type */
+		switch (o_ptr->sval)
+		{
+			case SV_RING_ACID:
+			{
+				text_out("acid resistance (20+d20 turns) and acid ball (70) every 50+d50 turns");
+				break;
+			}
+			case SV_RING_FLAMES:
+			{
+				text_out("fire resistance (20+d20 turns) and fire ball (80) every 50+d50 turns");
+				break;
+			}
+			case SV_RING_ICE:
+			{
+				text_out("cold resistance (20+d20 turns) and cold ball (75) every 50+d50 turns");
+				break;
+			}
+
+			case SV_RING_LIGHTNING:
+			{
+				text_out("electricity resistance (20+d20 turns) and electricity ball (85) every 50+d50 turns");
+				break;
+			}
+		}
+
+		return;
+	}
+
 	/* Require dragon scale mail */
 	if (o_ptr->tval != TV_DRAG_ARMOR) return;
 
