@@ -204,8 +204,9 @@ size_t vstrnfmt(char *buf, size_t max, cptr fmt, va_list vp)
 	char tmp[1024];
 
 
-	/* Mega-Hack -- treat "illegal" length as "infinite" */
-	if (!max) max = 32767;
+	/* Fatal error - no buffer length */
+	if (!max) quit("Called vstrnfmt() with empty buffer!");
+
 
 	/* Mega-Hack -- treat "no format" as "empty string" */
 	if (!fmt) fmt = "";
@@ -533,7 +534,6 @@ size_t vstrnfmt(char *buf, size_t max, cptr fmt, va_list vp)
 		/* Mega-Hack -- handle "capitalization" */
 		if (do_xtra)
 		{
-			/* Now append "tmp" to "buf" */
 			for (q = 0; tmp[q]; q++)
 			{
 				/* Notice first non-space */
@@ -569,6 +569,32 @@ size_t vstrnfmt(char *buf, size_t max, cptr fmt, va_list vp)
 }
 
 
+/*
+ * Add a formatted string to the end of a string
+ */
+void strnfcat(char *str, size_t max, size_t *end, cptr fmt, ...)
+{
+	size_t len;
+
+	va_list vp;
+
+	/* Paranoia */
+	if (*end >= max) return;
+
+	/* Begin the Varargs Stuff */
+	va_start(vp, fmt);
+
+	/* Build the string */
+	len = vstrnfmt(&str[*end], max - *end, fmt, vp);
+
+	/* End the Varargs Stuff */
+	va_end(vp);
+
+	/* Change the end value */
+	*end += len;
+}
+
+
 static char *format_buf = NULL;
 static size_t format_len = 0;
 
